DEFINE VARIABLE r1_rowid AS ROWID NO-UNDO.
DEFINE VARIABLE r2_rowid AS ROWID NO-UNDO.
DEFINE VARIABLE c1_rowid AS CHARACTER NO-UNDO.
DEFINE VARIABLE c2_rowid AS CHARACTER NO-UNDO.

/* let the user type in a customer2: */ 
PROMPT-FOR customer.unique2.

/* search the customer table in db: */
FIND customer USING unique2 NO-LOCK.

/* display customer's name: */
DISPLAY customer.name WITH SIDE-LABELS.

/* obtain the rowid of the record (if any): */
r1_rowid = ROWID(customer).
DISPLAY STRING(r1_rowid) SKIP(2) WITH SIDE-LABELS.

/* display it in user-readable format: */
c1_rowid = STRING(r1_rowid).
DISPLAY c1_rowid SKIP WITH SIDE-LABELS.

/* reverse conversion */
r2_rowid = TO-ROWID(c1_rowid).
DISPLAY STRING(r2_rowid) (r1_rowid = r2_rowid) SKIP
      WITH SIDE-LABELS. /* assert(r1_rowid == r2_rowid) */

/* reverse verification: */
c2_rowid = STRING(r2_rowid).
DISPLAY c2_rowid (c1_rowid = c2_rowid) SKIP(2)
      WITH SIDE-LABELS. /* assert(c1_rowid == c2_rowid) */



/* other test cases: */
DISPLAY "Other test cases:" SKIP.
RUN test0 (INPUT "0x",           INPUT ?).
RUN test0 (INPUT "0x00",         INPUT "0x00").
RUN test0 (INPUT "0x1",          INPUT ?).
RUN test0 (INPUT "0x01",         INPUT "0x01").
RUN test0 (INPUT "0X01",         INPUT ?).
RUN test0 (INPUT "0x0A",         INPUT ?).
RUN test0 (INPUT "0x1a",         INPUT "0x1a").
RUN test0 (INPUT "0x1234567890abcdef", INPUT "0x1234567890abcdef").
RUN test0 (INPUT "0xffffff0",    INPUT ?).
RUN test0 (INPUT "0x01ef",       INPUT "0x01ef").
RUN test0 (INPUT "0xffffff",     INPUT "0xffffff").
RUN test0 (INPUT "0x1000000",    INPUT ?).
RUN test0 (INPUT "0xbadrow",     INPUT ?).
RUN test0 (INPUT "badrow",       INPUT ?).

/* will fail because the first two 0s are discarded when parsing: */
RUN test0 (INPUT "0x0011223344556677", INPUT "0x0011223344556677").

/* will pass BUT it is incorect, just to prove previous declaration */
RUN test0 (INPUT "0x0011223344556677", INPUT "0x11223344556677").

/* largest rowid supported by implementation: */
RUN test0 (INPUT "0x7fffffffffffffff", INPUT "0x7fffffffffffffff").

/* OE valid, but not P2J, throw exception: */
RUN test0 (INPUT "0x8000000000000000", INPUT "0x8000000000000000").

/* OE valid, but not P2J, throw exception: */
RUN test0 (INPUT "0xffffffffffffffff", INPUT ?).

/* OE valid, but not P2J, throw exception: */
RUN test0 (INPUT "0x001122334455667788", INPUT "0x001122334455667788").

/* OE valid, but not P2J, throw exception: */
RUN test0 (INPUT "0xffffffffffffffffff", INPUT "0xffffffffffffffffff").

DISPLAY "<EOF>." SKIP.

PAUSE.
/* end of test program */


/* Test procedure, displays "Ok." or "Failed!" depending if the test */
/* value converted to rowid is equal to expected result */
/* Special handling if the test value converts to unknown value. */
PROCEDURE test0:
    DEFINE INPUT PARAMETER to_test AS CHARACTER.          /* test value */
    DEFINE INPUT PARAMETER ex_res  AS CHARACTER.      /* expected result*/

    DEFINE VARIABLE b AS CHARACTER FORMAT "x(42)". /* local char buffer */
    DEFINE VARIABLE r AS ROWID.      /* test value converted into rowid */
    DEFINE VARIABLE res AS LOGICAL.                      /* test result */

    r = TO-ROWID(to_test).
    b = "[" + to_test + "]=[" + (IF ex_res = ? THEN "?" ELSE ex_res) + "]".

    IF (r = ?) THEN
        res = (ex_res = ?).
    ELSE 
        IF (ex_res = ?) THEN
            res = FALSE.
        ELSE
            res = (ex_res = STRING(r)).

    DISPLAY b (IF res THEN "Ok." ELSE "Failed: " + STRING(r)) FORMAT "x(32)"
      SKIP WITH NO-LABELS NO-BOX.

    PAUSE.                                     /* let me see the result */
END PROCEDURE.

